@@ -30,7 +30,8 @@ import Control.Applicative ((<$>), (<*>))
 #endif
 import Control.Exception (bracket, throwIO)
 import Control.Monad (forM, when, (>=>))
-import Data.Array.IO (IOArray, newArray, readArray, writeArray)
+import Data.Array.Base (unsafeRead, unsafeWrite)
+import Data.Array.IO (IOArray, newArray)
 import qualified Data.ByteString.Char8 as BS
 import Data.IORef
 import Foreign.Marshal.Alloc
@@ -133,7 +134,7 @@ printDynamicTable DynamicTable{..} = do
     let !beg = off + 1
         !end = off + n
     tbl <- readIORef circularTable
-    es <- mapM (adj maxN >=> readArray tbl) [beg .. end]
+    es <- mapM (adj maxN >=> unsafeRead tbl) [beg .. end]
     let !ts = zip [1..] es
     mapM_ printEntry ts
     dsize <- readIORef dynamicTableSize
@@ -251,7 +252,7 @@ getEntries DynamicTable{..} = do
     off <- readIORef offset
     n <- readIORef numOfEntries
     table <- readIORef circularTable
-    let readTable i = adj maxN (off + i) >>= readArray table
+    let readTable i = adj maxN (off + i) >>= unsafeRead table
     forM [1 .. n] readTable
 
 copyEntries :: DynamicTable -> [Entry] -> IO ()
@@ -321,7 +322,7 @@ insertFront e DynamicTable{..} = do
     let i = off
         !dsize' = dsize + entrySize e
     !off' <- adj maxN (off - 1)
-    writeArray table i e
+    unsafeWrite table i e
     writeIORef offset off'
     writeIORef numOfEntries $ n + 1
     writeIORef dynamicTableSize dsize'
@@ -353,7 +354,7 @@ insertEnd e DynamicTable{..} = do
     table <- readIORef circularTable
     !i <- adj maxN (off + n + 1)
     let !dsize' = dsize + entrySize e
-    writeArray table i e
+    unsafeWrite table i e
     writeIORef numOfEntries $ n + 1
     writeIORef dynamicTableSize dsize'
     case codeInfo of
@@ -369,8 +370,8 @@ removeEnd DynamicTable{..} = do
     n <- readIORef numOfEntries
     !i <- adj maxN (off + n)
     table <- readIORef circularTable
-    e <- readArray table i
-    writeArray table i dummyEntry -- let the entry GCed
+    e <- unsafeRead table i
+    unsafeWrite table i dummyEntry -- let the entry GCed
     dsize <- readIORef dynamicTableSize
     let !dsize' = dsize - entrySize e
     writeIORef numOfEntries (n - 1)
@@ -386,7 +387,7 @@ toDynamicEntry DynamicTable{..} idx = do
     !off <- readIORef offset
     !didx <- adj maxN (idx + off - staticTableSize)
     !table <- readIORef circularTable
-    readArray table didx
+    unsafeRead table didx
 
 ----------------------------------------------------------------
 
